<?php
 /**
 * Asset Creation
 *
 * Provides functions to allow the creation of assets
 */	


require_once('linker.php');
require_once('cacheobject.php');
require_once('logging.php');

 /**
 * Creates an asset with no relationships
 *
 * This shouldn't be called by anything other than automated import scripts,
 * as it creates 'orphaned' (ie. unlinked) assets. If left unlinked, assets
 * like this can only be found via search.
 */	
function CreateAssetOrphaned($name, $desc)
{
	global $db;
	$db->query("INSERT INTO assets (name,descript,type) VALUES (".$db->quote($name).",".$db->quote($desc).",0)");
	
	$newid = $db->lastInsertId();
	
	LogEvent("asset",$newid,"Created");
		
	return $newid;
}

 /**
 * Creates an asset that appears after a parent
 *
 * This function creates a new asset and also places it in a relationship
 * with a given parent.
 */	
function CreateAssetAfter($name, $desc, $parent, $relationship)
{
	//echo 'Creating Asset '.$name.' Below '.$parent.' via '.$relationship;
	//return;
	global $db;
	$newid = CreateAssetOrphaned($name,$desc);	

	RebuildObjectCacheByID($newid);
	
	//echo 'New Asset ID: '.$newid;	
	LinkAssets($parent,$newid,$relationship."-D");
	
	if ($relationship == 3)
	{
		//Adjust this entry to make this item a TAG
		$db->query("UPDATE assets SET type=1 WHERE id=".$newid);
	}
	
	RebuildObjectCacheByID($newid);
	RebuildObjectCacheByID($parent);
	return $newid;	
}

 /**
 * Creates and inserts an asset
 *
 * This creation function builds a new asset that is part-way along an
 * existing relationship path. This requires that all relationships be
 * 'nudged' to make room.
 */	
function CreateAssetBefore($name, $desc, $parentid, $childid, $relationship)
{
	global $db;
	$newid = CreateAssetOrphaned($name,$desc);
	
	LinkInsert($newid, $parentid, $childid, $relationship);
	
	if ($relationship == 3)
	{
		//Adjust this entry to make this item a TAG
		$db->query("UPDATE assets SET type=1 WHERE id=".$newid);
	}
	
	RebuildObjectCacheByID($newid);
	RebuildObjectCacheByID($parentid);
	RebuildObjectCacheByID($childid);
	return $newid;		
}

?>