<?php
 /**
 * Simple Authentication System
 *
 * This authentication system uses the internal database to store user
 * information.
 */	
 
class AuthSystem
{
	//TRUE if password changes are supported by this source
	public $passchangesupported = true;
	
	//TRUE if password resets are supported by this source
	public $passresetsupported = true;
	
	private $dbx = null;
	
	public function GetDatabase()
	{
		if ($this->dbx != null)
		{
			return $this->dbx;
		}
		
		global $db;	
		global $sitemode;
		global $sitefolder;
		global $installfolder;
		
		$this->dbx = $db;
		if (($sitemode == 'combined') || ($sitemode == 'integrated'))
		{
			//Change database to handle centralised authentication...
			include($installfolder.'/sites/default/config/dbsettings.php');
			
			$srvr = "localhost";
			if ($dbserver != "") $srvr = $dbserver;
			
			$dbcommandstring = $dbtype.":host=".$srvr.';dbname='.$dbname.";charset=utf8";
			
			if (isset($dboptions))	
				$this->dbx = new PDO($dbcommandstring,$dbuser,$dbpass,$dboptions);	
			else
				$this->dbx = new PDO($dbcommandstring,$dbuser,$dbpass);
			
			include($sitefolder.'/config/dbsettings.php');
		}
		
		return $this->dbx;
	}
	
	 /**
	 * Authenticate a user by username and password
	 *
	 * This function returns 'true' if the username and password are valid,
	 * and 'false' if they are invalid.
	 */	
	public function AuthUser($name, $pass)
	{
		$dbx = $this->GetDatabase();
		
		//Simple database-query style lookup
		$query = $dbx->query("SELECT users.userid,basegroup FROM users WHERE username=".$dbx->quote($name)." AND pwd=PASSWORD(".$dbx->quote($pass).")");
		
		//echo "SELECT users.userid,basegroup FROM users WHERE username=".$db->quote($name)." AND pwd=PASSWORD(".$db->quote($pass).")";
							
		if ($query === FALSE)
		{
			return false;
		}
		
		if($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{
			$_SESSION['userid'] = $row['userid'];
			$_SESSION['usertoken'] = $row['userid'];
			$_SESSION['userread'] = true;
			$_SESSION['useredit'] = false;
			$_SESSION['usercreate'] = false;
			$_SESSION['userdelete'] = false;
			$_SESSION['useradmin'] = false;
			$_SESSION['useroverlord'] = false;
			
			if ($row['basegroup'] == 'Editor')
			{
				$_SESSION['useredit'] = true;
				$_SESSION['usercreate'] = true;
				$_SESSION['userdelete'] = true;
			}
			if ($row['basegroup'] == 'Builder')
			{
				$_SESSION['useredit'] = true;
				$_SESSION['usercreate'] = true;
				$_SESSION['userdelete'] = true;
				$_SESSION['useradmin'] = true;
			}
			if ($row['basegroup'] == 'Superuser')
			{
				$_SESSION['useredit'] = true;
				$_SESSION['usercreate'] = true;
				$_SESSION['userdelete'] = true;
				$_SESSION['useradmin'] = true;
				$_SESSION['usersuper'] = true;
			}
			
			return true;
		}	
		
		return false;
	}
	
	 /**
	 * Confirm User Login via Token
	 *
	 * Confirms the user password by their 'token' (which may simply be the
	 * username, or may be an LDAP address etc.) and their password.
	 */	
	public function ConfirmUser($token, $pass)
	{
		$dbx = $this->GetDatabase();
		
		//Simple database-query style lookup
		$query = $dbx->query("SELECT users.userid,basegroup FROM users WHERE token=".$dbx->quote($token)." AND pwd=PASSWORD(".$dbx->quote($pass).")");				
							
		if ($query === FALSE)
		{
			return false;
		}
		
		if($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{						
			return true;
		}	
		
		return false;
	}
	
	/**
	 * Prepare Reset Codes (by name)
	 *
	 * Determines the users ID before passing it to PasswordResetByID
	 */	
	public function PasswordResetRequest($name,$email)
	{		
		$dbx = $this->GetDatabase();
		$query = $dbx->query("SELECT users.userid, email FROM users WHERE email=".$dbx->quote($email)." AND username=".$dbx->quote($name));
		
		//echo "SELECT users.userid, email FROM users WHERE email=".$db->quote($email)." AND username=".$db->quote($name);
	
		if ($query === FALSE)
		{
			print "Failed To Locate Target";
			return false;
		}
		
		if($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{
			return $this->PasswordResetByID($row['userid'],$row['email']);			
		}	
	}
		
	/**
	 * Begin Password Reset - Phase 1
	 *
	 * Establishes the unique code that will be used to validate a password
	 * reset attempt and emails out the password reset link to the user.
	 */	
	function PasswordResetByID($id,$email)
	{
		//global $db;		
		$dbx = $this->GetDatabase();
		
		//Generate One-Time Code
		$bits = openssl_random_pseudo_bytes(12);
		$hex = bin2hex($bits);
		$final = sha1($hex);
		
		$dbx->query("DELETE FROM userdata WHERE userid=".$id." AND property IN ('resetcode','resetexpires')");
		
		$date = new DateTime();
		$date->add(new DateInterval('PT3H'));
		
		$dbx->query("INSERT INTO userdata (userid, property, value) VALUES (".$id.",'resetcode','".$final."'),(".$id.",'resetexpires','".$date->format('Y-m-d H:i:s')."')");
		
		require_once($_SERVER['DOCUMENT_ROOT'].'/include/mail.php');
		
		//echo 'Sending Message';

		$mail = PrepareMail($email);
		
		$targeturl = 'http://'.$_SERVER['HTTP_HOST'].'/users/resetpwd.php?resetcode='.$final;
		
		$mail->Subject = 'ARDI Password Reset Request';
		$mail->Body    = 'Someone has requested a password reset from an <b>ARDI</b> server.<br/><br/>To reset your password, please click the link below...<br/><br/><a href="'.$targeturl.'">'.$targeturl.'</a><br/>';
		$mail->AltBody = "Someone has requested a password reset from ARDI\r\n\r\nTo reset your password, please visit...\r\n\r\n".$targeturl;
		
		if(!$mail->send()) {
 		   return "Mail Error";
		}
		
		return "OK";
	}
	
	/**
	 * Change password w/ confirmation
	 *
	 * Changes the user password - requires the existing token before
	 * the reset can be completed.
	 */	
	public function PasswordChange($token, $currentpassword, $newpassword)
	{
		$dbx = $this->GetDatabase();
		$dbx->query("UPDATE users SET pwd=PASSWORD(".$dbx->quote($newpassword).") WHERE token=".$dbx->quote($token)." AND pwd=PASSWORD(".$dbx->quote($currentpassword).")");
		
		//echo "UPDATE users SET pwd=PASSWORD(".$db->quote($newpassword).") WHERE token=".$db->quote($token)." AND pwd=PASSWORD(".$db->quote($currentpassword).")";
	}
	
	/**
	 * Change password w/o confirmation
	 *
	 * Changes the password of a user given only their token and the new
	 * password
	 */
	public function PasswordReset($code,$newpassword)
	{
		//global $db;		
		$dbx = $this->GetDatabase();	
		//Get the user that matches this one-time code...
		
		$query = $dbx->query("SELECT userid FROM userdata WHERE value=".$dbx->quote($code));
		
		//echo "SELECT userid FROM userdata WHERE value=".$db->quote($code);
	
		if ($query === FALSE)
		{
			//echo "Can't find code '".$code."'";
			return false;
		}
		
		if($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{
			$targetid = $row['userid'];
			//$targetemail = $row['email'];
		}	
		
		//Check Expiry Date
		
		$date = new DateTime();
		$cmpdate = $date->format('Y-m-d H:i:s');
		
		$query = $dbx->query("SELECT value FROM userdata WHERE userid=".$targetid." AND property='resetexpires'");
	
		if ($query === FALSE)
		{
			//echo "Can't find expiry date!";			
			$dbx->query("DELETE FROM userdata WHERE userid=".$targetid." AND property IN ('resetcode','resetexpires')");
			return false;
		}
		
		if($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{
			$expirydate = $row['value'];
		}	
		
		if ($expirydate < $cmpdate)
		{
			$dbx->query("DELETE FROM userdata WHERE userid=".$targetid." AND property IN ('resetcode','resetexpires')");
			return 'Expired';
		}
				
		$dbx->query("UPDATE users SET pwd=PASSWORD(".$dbx->quote($newpassword).") WHERE userid=".$targetid);
		
		$dbx->query("DELETE FROM userdata WHERE userid=".$targetid." AND property IN ('resetcode','resetexpires')");
		
		unset($_SESSION['userid']);
		
		return "OK";
		
	}
}


?>