<?php
/**
 * User Activity Storage
 * 
 * These functions allow the system to store favourites and recent activity
 *
 * @author Steven Harding
 */	
 
 /**
 * Records a Users Access to an Asset
 *
 * Saves a user setting to record that the current user has accessed an
 * asset recently. 
 */	
function MarkUserAccess($assetid)
{
	if (!isset($_SESSION['userid'])) return false;
	global $db;	
	
	$query = $db->query("SELECT value FROM userdata WHERE userid=".$_SESSION['userid']." AND property='recent'");
	
	if ($row = $query->fetch(PDO::FETCH_ASSOC)) 
	{
		$faves = $row['value'];
	}		

	$faveset = array();
	if (isset($faves))
	{
		$faveset = explode('|',$faves);
		if (!in_array($assetid,$faveset))
		{
			if (count($faveset) > 5)
			{
				$faveset = array_splice($faveset,0,4);				
			}
			array_unshift($faveset,$assetid);
		}
		else
		{
			//Swap Array Elements....
			for($x=0;$x<count($faveset);$x++)
			{
				if ($faveset[$x] == $assetid)
				{
					if ($x == 0) return;
					for($n=$x;$n > 0;$n--)
					{
						$faveset[$n] = $faveset[$n-1];
					}
					$faveset[0] = $assetid;
				}
			}
		}		
		
		$final = implode('|',$faveset);
		if ($final[strlen($final)-1] == '|')
		{
			$final = substr($final,0,strlen($final)-1);
		}
		
		$db->query("UPDATE userdata SET value=".$db->quote($final)." WHERE property='recent' AND userid=".$_SESSION['userid']);
	}
	else
	{
		$db->query("INSERT userdata SET userid=".$_SESSION['userid'].", property='recent', value='".$assetid."'");
	}
}

 /**
 * Determine if the asset is a Favourite
 *
 * Returns 'true' if the asset is a favourite of the current users.
 *
 * NOTE: Using this function also marks the asset as visited, so if you
 * don't intend for this to happen, please check for favourites using
 * a regular SQL query.
 */	
function IsFave($assetid)
{
	if (!isset($_SESSION['userid'])) return false;
	global $db;
	$query = $db->query("SELECT value FROM userdata WHERE userid=".$_SESSION['userid']." AND property='faves'");

	if ($row = $query->fetch(PDO::FETCH_ASSOC)) 
	{
		$faves = $row['value'];
	}	
	
	MarkUserAccess($assetid);
	
	if (isset($faves))
	{
		$faveset = explode('|',$faves);
		if (in_array($assetid,$faveset))
		{
			return true;
		}
	}
	return false;
}

 /**
 * Toggles Asset Favourite Status
 *
 * Toggles the favourite status of this object, returning the NEW status
 * of the object (in case it was originally unknown).
 */	
function ToggleFave($assetid)
{
	if (!isset($_SESSION['userid'])) return false;
	global $db;	
	$query = $db->query("SELECT value FROM userdata WHERE userid=".$_SESSION['userid']." AND property='faves'");

	if ($row = $query->fetch(PDO::FETCH_ASSOC)) 
	{
		$faves = $row['value'];
	}	
	
	$resp = false;
	
	if (isset($faves))
	{
		$faveset = explode('|',$faves);
		if (!in_array($assetid,$faveset))
		{
			//echo 'Asset Not In Faves - Adding';
			if (count($faveset) > 20)
			{
				$faveset = array_splice($faveset,0,19);
			}
			array_unshift($faveset,$assetid);
			$resp = true;
		}
		else
		{
			for($x=0;$x<count($faveset);$x++)
			{
				if ($faveset[$x] == $assetid)
				{
					unset($faveset[$x]);
					$faveset = array_values($faveset);
					break;
				}
			}				
		}		
		$final = implode('|',$faveset);
		if (($final != "") && ($final[strlen($final)-1] == '|'))
		{
			$final = substr($final,0,strlen($final)-1);
		}
		
		$db->query("UPDATE userdata SET value=".$db->quote($final)." WHERE property='faves' AND userid=".$_SESSION['userid']);				
	}
	else
	{
		$db->query("INSERT userdata SET userid=".$_SESSION['userid'].", property='faves', value='".$assetid."'");
	}
	
	return $resp;
}

?>