<?php

class AssetProperty
{
	public $name;
	public $type;
	public $id;
	public $origin;
	public $origintype;
	public $value;
}

class Media
{
	public $id;
	public $name;
	public $type;
	public $path;
	public $category;
	public $asset;
}

if (!function_exists('LogEvent'))
{
	include($installfolder.'/include/logging.php');
}
		
if (!function_exists('RebuildObjectCacheByID'))
{
	include($installfolder.'/include/cacheobject.php');
}

if (!function_exists('GetActiveSources'))
{
	include($_SERVER['DOCUMENT_ROOT'].'/include/isocheck.php');
}
//Provides an interface to get information about particular assets.

class Asset
{
	public $id = 0;
	public $name = "";
	public $description = "";
	public $tag = false;
	public $root = false;
	
	private $working = array();
	private $found = array();
	private $visited = array();
	
	public function Asset($id = 0)
	{
		if ($id != 0) 
		{
			$this->id = $id;
			$this->Get();
		}
	}
	
	public function Get()
	{
		global $sitefolder;
		global $asset;
		
		if (!isset($asset[$this->id]))
		{
			require($sitefolder.'/cache/relationships/'.$this->id.'.inc');
		}
		
		$this->name = $asset[$this->id]['name'];
	}
	
	public function GetFullName()
	{
		global $db;
		
		//Get all Location tags...
		$qry = 'SELECT parentasset,depth,name FROM assetrelclosures JOIN assets ON (assetrelclosures.parentasset = assets.id) WHERE childasset='.$this->id.' AND relationship=2 ORDER BY depth DESC';
		$query = $db->query($qry);
		
		$nm = $this->name;
		while($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{
			//echo 'Processing Path Element '.$row['name'];
			if ($row['depth'] <= 0) continue;
			if (strpos($nm,$row['name']) !== FALSE)
			{
				continue;
			}
			if ($row['name'][strlen($row['name'])-1] != 's')
			{
				$nm = $row['name'].' '.$nm;
			}
		}
		return $nm;
	}
	
	public function Load()
	{
		$this->Get();		
		
		global $db;
		$qry = 'SELECT descript,type FROM assets WHERE id='.$this->id;
		$query = $db->query($qry);
		
		if($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{
			$this->description = $row['descript'];
			//echo 'Asset Type: '.$row['type'];
			if ($row['type'] == 2) $this->root = true;
			if ($row['type'] == 1) $this->tag = true;
		}
	}
	
	public function GetLocalProperties($followexternal = true)
	{
		$this->found = array();
		global $asset;
		global $sitefolder;
		global $db;
		$props = array();
		
		if (!isset($asset[$this->id]))
		{
			require_once($sitefolder.'/cache/relationships/'.$this->id.'.inc');
		}
		
		$qry = 'SELECT propertyid,typename,name,value,assetvalues.assetid FROM assetvalues JOIN properties ON (properties.id = assetvalues.propertyid) WHERE assetvalues.assetid = '.$this->id;
		
		$query = $db->query($qry);
		
		while($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{	
			$prp = new AssetProperty();
			$prp->name = $row['name'];
			$prp->type = $row['typename'];
			$prp->id = $row['propertyid'];
			$prp->origin = $row['assetid'];
			$prp->origintype = 'local';
			$this->found[$row['propertyid']] = true;
			//echo 'Found '.$row['propertyid'];
			$props[] = $prp;
			
			//echo 'Adding Property '.$prp->name;
		}
		
		$qry = 'SELECT propertyid,typename,name,value,assetvalues.assetid FROM assetrelclosures JOIN assetvalues ON (assetvalues.assetid = assetrelclosures.parentasset) JOIN properties ON (properties.id = assetvalues.propertyid) WHERE assetrelclosures.childasset = '.$this->id.' AND assetrelclosures.relationship=3 ORDER BY depth DESC';
		
		$query = $db->query($qry);
		
		//echo $qry;
		
		while($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{
			//echo 'Checking For An Existing '.$row['propertyid'];
			if (isset($this->found[$row['propertyid']]))
			{
				//echo 'Rejected '.$row['propertyid'];
				continue;
				
			}
			
			$prp = new AssetProperty();
			$prp->name = $row['name'];
			$prp->type = $row['typename'];
			$prp->id = $row['propertyid'];
			$prp->origin = $row['assetid'];
			$prp->origintype = 'template';
			$this->found[$row['propertyid']] = true;
			$props[] = $prp;
		}
		
		//Check for a 'Monitors' relationship...
		if (isset($asset[$this->id]["relationships"][6]))
		{
			global $relationships;
			require_once($sitefolder.'/cache/relationships/all.inc');
	
			foreach($asset[$this->id]["relationships"][6] as $r)
			{
				if ($r['upstream'] == true)
				{
					//This asset is monitored by another!
					$tg = $r['target'];
					if (!isset($asset[$tg]))
					{
						require($sitefolder.'/cache/relationships/'.$tg.'.inc');
					}
					
					$passlist = $this->GetFlowingPropertyIDs();
					
					foreach($asset[$tg]["values"] as $key => $val)
					{
						//Check if this is a flowing property.
						if (!in_array($key,$passlist)) continue;
						
						$prp = new AssetProperty();
						$prp->name = $val['name'];
						$prp->type = $val['type'];
						$prp->id = $key;
						$prp->origin = $tg;
						$prp->origintype = 'monitors';
						$props[] = $prp;
						$this->found[$key] = true;
					}
				}
			}	
		}
		
		if ($followexternal == true)
		{
			if (isset($asset[$this->id]["ref"]))
			{
				//Handle External References...
				$gathering = array();
				foreach($asset[$this->id]["ref"] as $rf)
				{
					$bits = explode('-',$rf);
					
					if (!isset($gathering[$bits[0]]))
					{
						$gathering[$bits[0]] = array();
					}
					$gathering[$bits[0]][] = $bits[1];
				}
				
				foreach($gathering as $k => $v)
				{
					$other = new Asset($k);
					$prps = $other->GetLocalProperties(false);
					foreach($v as $n)
					{
						foreach($prps as $p)
						{
							if ($p->id == $n)
							{
								$prp = new AssetProperty();
								$prp->name = $p->name;
								$prp->type = $p->type;
								$prp->id = $p->id;
								$prp->origin = $k;
								$prp->origintype = 'reference';
								$props[] = $prp;
							}
						}												
					}
				}
			}
		}
		
		return $props;
	
	}

	public function GetLocalMeasurements()
	{
		$this->found = array();
		global $asset;
		global $sitefolder;
		
		if (!isset($asset[$this->id]))
		{
			require_once($sitefolder.'/cache/relationships/'.$this->id.'.inc');
		}
		
		$ids = $this->GetTagIDs();
		if (!in_array($this->id,$ids))
		{
			array_unshift($ids,$this->id);
		}
		
		//Get local properties...
		$props = array();
		foreach($asset[$this->id]["values"] as $key => $val)
		{
			$prp = new AssetProperty();
			$prp->name = $val['name'];
			$prp->type = $val['type'];
			$prp->id = $key;
			$prp->origin = $this->id;
			$prp->origintype = 'local';
			$this->found[$key] = true;
			$props[] = $prp;
		}
		
		//Get tagged properties...
		
		foreach($ids as $id)
		{
			if (!isset($asset[$id]))
			{
				require_once($sitefolder.'/cache/relationships/'.$id.'.inc');
			}
			
			//Get tagged item properties...
			foreach($asset[$id]["values"] as $key => $val)
			{
				if (isset($this->found[$key])) continue;
			
				$prp = new AssetProperty();
				$prp->name = $val['name'];
				$prp->type = $val['type'];
				$prp->id = $key;
				$prp->origin = $id;
				$prp->origintype = 'template';
				$this->found[$key] = true;
				$props[] = $prp;
			}
		}
		
		//Check for a 'Monitors' relationship...
		if (isset($asset[$this->id]["relationships"][6]))
		{
			global $relationships;
			require_once($sitefolder.'/cache/relationships/all.inc');
	
			foreach($asset[$this->id]["relationships"][6] as $r)
			{
				if ($r['upstream'] == true)
				{
					//This asset is monitored by another!
					$tg = $r['target'];
					if (!isset($asset[$tg]))
					{
						require_once($sitefolder.'/cache/relationships/'.$tg.'.inc');
					}
					
					$passlist = $this->GetFlowingPropertyIDs();
					
					foreach($asset[$tg]["values"] as $key => $val)
					{
						//Check if this is a flowing property.
						if (!in_array($key,$passlist)) continue;
						
						$prp = new AssetProperty();
						$prp->name = $val['name'];
						$prp->type = $val['type'];
						$prp->id = $key;
						$prp->origin = $tg;
						$prp->origintype = 'monitors';
						$props[] = $prp;
						$this->found[$key] = true;
					}
				}
			}	
		}
		
		return $props;
	}
	
	public function HasMeasurements()
	{
		$arr = $this->GetLocalMeasurements();
		if (count($arr) > 0) return true;
		$arr = $this->GetSecondaryProperties();
		if (count($arr) > 0) return true;
		return false;
	}
	
	//Returns a list of secondary (flowed) properties
	public function GetSecondaryProperties()
	{
		$this->found = array();
		$this->GetLocalProperties();
		$this->FindSecondaries($this->id,0);
		return $this->working;
	}
	
	//Returns a list of all properties
	public function GetProperties()
	{
		$this->found = array();
		$this->working = $this->GetLocalProperties();
		//print_r($this->working);
		$this->FindSecondaries($this->id,0);
		return $this->working;
	}
	
	public function GetMedia()
	{
		global $db;
		global $sitefolder;
		global $siteroot;
		
		$medialist = array();
		
		$subfolder = substr($sitefolder,strlen($_SERVER['DOCUMENT_ROOT'])).'/media/assets';
		$fldr = $subfolder.'/'.$this->id;
				
		$names = array();
		$any = false;
		
		$query = $db->query('SELECT id,mediatype,name,category,path FROM mediatags JOIN mediafiles ON (mediafiles.id = mediatags.mediaid) WHERE assetid='.$this->id.' ORDER BY category ASC, name ASC');
		while($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{
			if (in_array($row['name'],$names)) continue;
			$names[] = $row['name'];
			
			$mc = $row['category'];
			if ($mc == "") $mc = "Misc";
			
			if ($row['mediatype'] == 'link')
			{
				$it = new Media();
				$it->id = $row['id'];
				$it->name = $row['name'];
				$it->path = $row['path'];
				$it->asset = $this->id;
				$it->category = $mc;
				
				$medialist[] = $it;
			}
			else
			{
				$it = new Media();
				$it->id = $row['id'];
				$it->name = $row['name'];
				$it->path = $siteroot.'/media/assets/'.$this->id.'/'.$row['path'];
				$it->asset = $this->id;
				$it->category = $mc;
				
				$medialist[] = $it;	
			}
		}
		
		$query = $db->query('SELECT id,mediatype,name,category,path,depth,mediatags.assetid FROM assetrelclosures JOIN mediatags ON (mediatags.assetid = assetrelclosures.parentasset) JOIN mediafiles ON (mediafiles.id = mediatags.mediaid) WHERE assetrelclosures.childasset='.$this->id.' AND assetrelclosures.relationship = 3 ORDER BY depth DESC, category ASC, name ASC');
		while($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{
			if (in_array($row['name'],$names)) continue;
			$names[] = $row['name'];
			
			$mc = $row['category'];
			if ($mc == "") $mc = "Misc";
				
			$fldr = $subfolder.'/'.$row['assetid'];
			$relative = $fldr;
			
			if ($row['mediatype'] == 'link')
			{
				$it = new Media();
				$it->id = $row['id'];
				$it->name = $row['name'];
				$it->path = $row['path'];
				$it->asset = $row['assetid'];
				$it->category = $mc;
				
				$medialist[] = $it;
			}
			else
			{
				$it = new Media();
				$it->id = $row['id'];
				$it->name = $row['name'];
				$it->path = $siteroot.'/media/assets/'.$row['assetid'].'/'.$row['path'];
				$it->asset = $row['assetid'];
				$it->category = $mc;
				
				$medialist[] = $it;	
			}
		}
		return $medialist;
	}
	
	//Returns a list of items that this asset is tagged as (includes all children)
	public function GetTagIDs()
	{
		global $db;
		$qry = 'SELECT parentasset FROM assetrelclosures WHERE childasset='.$this->id.' AND relationship=3 ORDER BY depth DESC';
		$query = $db->query($qry);
		
		$valueset = array();
		while($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{
			if ($row['parentasset'] == $this->id) continue;
			$valueset[] = $row['parentasset'];
		}
		
		//echo '<strong>Tagged As:</strong> '.implode(',',$valueset);
		
		return $valueset;
	}
	
	//Returns a list of items that this asset is tagged as (includes all children)
	public function GetDirectTagIDs()
	{
		global $db;
		$qry = 'SELECT parentasset FROM assetrelationships WHERE childasset='.$this->id.' AND relationship=3';
		$query = $db->query($qry);
		
		$valueset = array();
		while($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{
			if ($row['parentasset'] == $this->id) continue;
			$valueset[] = $row['parentasset'];
		}
		
		//echo '<strong>Tagged As:</strong> '.implode(',',$valueset);
		
		return $valueset;
	}
	
	//Returns the ID numbers of the properties that flow through this asset.
	public function GetFlowingPropertyIDs()
	{
		global $sitefolder;
		global $relationships;
		require_once($sitefolder.'/cache/relationships/all.inc');
		
		$passlist = array();
		foreach($relationships as $k => $v)
		{
			if (isset($v['passes']))
			{
				foreach($v['passes'] as $itm)
				{
					if (!in_array($itm,$passlist))
					{
						$passlist[] = $itm;
					}
				}
			}
		}
		
		return $passlist;
	}
	
	//For any properties WITHOUT values, it loads the values.
	public function GetValues(&$props)
	{
		global $db;
	
		$assetlist = array();
		foreach($props as $p)
		{
			if (!in_array($p->origin,$assetlist)) $assetlist[] = $p->origin;
		}
		
		if (count($assetlist) == 0) return;
	
		$qry = 'SELECT assetid, propertyid, value FROM assetvalues WHERE assetvalues.assetid IN ('.implode(',',$assetlist).')';
		$query = $db->query($qry);
		
		$valueset = array();
		while($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{
			$valueset[] = $row;
		}
		
		if (!function_exists('LinkedData'))
		{
			global $installfolder;
			require($installfolder.'/include/dynamicdata.php');
		}
		
		foreach($valueset as $row)
		{
			for($x=0;$x<count($props);$x++)
			{
				if (($props[$x]->origin != $row['assetid']) || ($props[$x]->id != $row['propertyid'])) continue;
				if ($props[$x]->value != "") continue;
				
				$basevalue = $row['value'];
				$module=$props[$x]->type;
				
				$value = array();
				$bits = explode('|',$basevalue);		

				$k = $props[$x]->id;
								
				$modulefile = $_SERVER['DOCUMENT_ROOT'].'/modules/inputs/'.strtolower($module).'.php';
				if (file_exists($modulefile))
				{
					//Import module...
					include ($modulefile);
							
					for($q=0;$q<count($inputs);$q++)
					{
						$isdynamic = false;
						if ((!isset($bits[$q])) || ($bits[$q] == "")) 
						{
							if (isset($inputs[$q]['default']))
								$value[$inputs[$q]['name']] = LinkedData($props[$x]->origin,$k,$inputs[$q]['name'],$inputs[$q]['default'],$isdynamic);
							else
								$value[$inputs[$q]['name']] = LinkedData($props[$x]->origin,$k,$inputs[$q]['name'],'',$isdynamic);	

							/*if ($props[$x]->type == "STATE") 
							{
								echo 'Values = '.$value[$inputs[$q]['name']];
							}*/
						
							continue;
						}
						$value[$inputs[$q]['name']] = LinkedData($props[$x]->origin,$k,$inputs[$q]['name'],$bits[$q],$isdynamic);
						
						if ($isdynamic != false)
						{
							if (!isset($value['_dynamics']))
							{
								$value['_dynamics'] = array();
							}
							$value['_dynamics'][] = $inputs[$q]['name'];
						}
						
					}
				}
				
				
				$val = $basevalue;
				if ($props[$x]->type == "MEASUREMENT") 
				{
					$val = $value['measurement'];
					if (is_numeric($val))
					{
						$val = number_format($val,$value['places']);
					}
					
					$val = $val.$value['units'];
					if (trim($value['measurement']) == '^')
					{
						$val = "Error";
					}	
					
					$value['live'] = $val;
				}
				if ($props[$x]->type == "STATE") 
				{
					$val = $value['state'];					
					if (trim($val) == '^')
					{
						$val = "Error";
					}	
					if ($val == "0") $val = $value['off'];
					if ($val == "1") $val = $value['on'];
					
					$value['live'] = $val;
				}
				
				$props[$x]->value = $value;
			}
		}
	}
	
	public function GetRelationships()
	{
		global $asset;
		if (!isset($asset[$this->id]))
		{
			require($sitefolder.'/cache/relationships/'.$this->id.'.inc');
		}
		
		return $asset[$this->id]['relationships'];
	}
	
	public function GetIsolationStatus()
	{
		$info = array();
		
		global $results;
		$results = array();
		
		$state='Unknown';
		
		if (GetActiveSources($this->id) !== FALSE)
		{	
			$state = "Not Isolated";
			if (count($results) == 0)
			{
				$state = 'Isolated';
			}
			else
			{
				foreach($results as $res) 
				{
					if ($res['id'] == $this->id)
					{
						$state = 'Unknown';
						$fp = array();
						$fp['name'] = $res['name'];
						$fp['id'] = $this->id;
						$info['failure'][] = $fp;
					}
				}
				
				foreach($results as $res)
				{
					if (!isset($info['failure']))
					{
						$info['failure'] = array();
					}
					$fp = array();
					$fp['name'] = $res['name'];
					$fp['id'] = $res['id'];
					$info['failure'][] = $fp;
				}
			}
			
			//echo $content.'<status>'.$state.'</status>';
		}
		
		$info['state'] = $state;
		return $info;
	}
	
	private function FindSecondaries($id,$rels)
	{
		global $db;
		global $asset;
		global $relationships;
		global $sitefolder;
		
		if (isset($this->visited[$id]))
		{
			//echo 'I Already Visited '.$asset[$id]['name'].' - Stop Here!<br/>';
			return;
		}
		$this->visited[$id] = true;
		
		if (!isset($asset[$id]))
		{		
			require($sitefolder.'/cache/relationships/'.$id.'.inc');
		}
				
		if (!isset($asset[$id]))
		{		
			return;
		}
		
		//echo 'Secondary Scan - '.$asset[$id]['name'].'...<br/>';
		
		foreach($asset[$id]["values"] as $k => $v)
		{			
			if (($rels != 0) && ($rels != 6))
			{
				if (!isset($relationships[$rels]["passes"])) continue;
				if (!in_array($k,$relationships[$rels]["passes"])) continue;
			}	
			if (($v['type'] == "MEASUREMENT") || ($v['type'] == "STATUS"))
			{
				//echo 'Located Measurement!';
				if (!isset($this->found[$k]))
				{
					if ($rels != 0)
					{ 
						//OK - load some value data here....
						
						//Do something here...
						$prp = new AssetProperty();
						$prp->name = $v['name'];
						$prp->type = $v['type'];
						$prp->id = $k;
						$prp->value = "";//$val;
						$prp->origin = $id;
						$prp->origintype = 'secondary';
						
						$this->working[] = $prp;
						
					}
					$this->found[$k] = true;
				}				
			}			
		}
		
		if ($rels == 0)
		{			
			//echo 'Scanning All Physical Relationshps!';
			
			
			foreach($asset[$id]["relationships"] as $k => $v)
			{
				if ($relationships[$k]['type'] != 'physical') continue;
				//echo 'Scan '.$k;
				foreach($v as $rel)
				{
					if ($rel['upstream'] == true)
					{				
						if (isset($relationships[$k]['passes']))
						{
							$alldone = true;
							foreach($relationships[$k]['passes'] as $propallowed)
							{
								if (!isset($found[$propallowed]))
								{
									//echo 'Blanks!';
									$alldone = false;
								}
							}
							if (!$alldone) $this->FindSecondaries($rel["target"],$k);
						}
					}
				}
			}
		}
		else
		{
			//echo 'Subscanning Relationship '.$rels;
			if (isset($asset[$id]["relationships"][6]))
			{
				foreach($asset[$id]["relationships"][6] as $rl)
				{
					if ($rl["upstream"] == true)
					{
						//echo 'Following Monitors Relationship To '.$rl['target'];
						$this->FindSecondaries($rl['target'],$rels);
					}
				}
			}
			
			if (isset($asset[$id]["relationships"][$rels]))
			{
				//echo 'Relationship Direction: '.$asset[$id]["relationships"][$rels]["upstream"];
				foreach($asset[$id]["relationships"][$rels] as $rl)
				{
					if ($rl["upstream"] == true)
					{
						//echo 'Subscan...';
						//Hold on...are there any properties LEFT?
						$alldone = true;
						foreach($relationships[$rels]['passes'] as $propallowed)
						{
							if (!isset($found[$propallowed]))
							{
								$alldone = false;
							}
						}								
						
						//echo $alldone;
						
						if ($alldone == false) $this->FindSecondaries($rl["target"],$rels);
					}
				}
			}
		}
	}	
	
	public function SetProperty($id, $value)
	{
		global $db;
		$db->query('INSERT INTO assetvalues (assetid, propertyid, value) VALUES ('.$this->id.','.$id.",".$db->quote($value).") ON DUPLICATE KEY UPDATE value=".$db->quote($value));
		//echo 'INSERT INTO assetvalues (assetid, propertyid, value) VALUES ('.$this->id.','.$id.",".$db->quote($value).") ON DUPLICATE KEY UPDATE value=".$db->quote($value);
	}
	
	public function UpdateCache()
	{
		if (!function_exists('RebuildObjectCacheByID'))
		{
			include($installfolder.'/include/cacheobject.php');
		}
		RebuildObjectCacheByID($this->id);
	}
	
	public function Delete()
	{
		global $db;
		global $sitefolder;
		
		$id = $this->id;
		$debug = false;
		
		$toupdate = array();
					
		$itemid = $id;
		
		
		LogEvent("asset",$id,"Deleted Asset");
		
		include($sitefolder.'/cache/relationships/'.$itemid.'.inc');
		//echo 'Starting Delete...';
		
		foreach($asset[$itemid]['relationships'] as $k => $v)
		{
			$bindparent = array();
			$bindchild = array();
			foreach($v as $rel)
			{
				if ($rel['upstream'] == true)
				{
					$bindparent[] = $rel['target'];
					if (!in_array($rel['target'],$toupdate))
					{
						$toupdate[] = $rel['target'];
					}
				}
				else
				{
					$bindchild[] = $rel['target'];
					if (!in_array($rel['target'],$toupdate))
					{
						$toupdate[] = $rel['target'];
					}
				}
			}
			
			foreach($bindparent as $par)
			{
				foreach($bindchild as $chld)
				{								
					//Remap each relationship on the TARGET object to its children (this effectively removes the object from the existing relationship paths).
													
					$query = "INSERT IGNORE INTO assetrelationships (parentasset, childasset, relationship) VALUES (".$par.','.$chld.','.$k.")";
					if ($debug == false)
						$db->query($query);
					else
						echo $query;									
						
					/*$prevdepth = 4;
					//Fetch the existing depth before we replace it...
					$query = $db->query("SELECT depth FROM assetrelclosures WHERE childasset=".$chld." AND parentasset=".$par." AND relationship=".$k);
					if ($query !== FALSE)
					{
						while($row = $query->fetch(PDO::FETCH_ASSOC)) 
						{
							$prevdepth = $row['depth'];
						}
					}
						
					$query = "INSERT IGNORE INTO assetrelclosures (parentasset, childasset, relationship,depth) VALUES (".$par.','.$chld.','.$k.","")";
					if ($debug == false)
						$db->query($query);
					else
						echo $query;*/									
					
					$query = "DELETE FROM assetrelclosures WHERE parentasset=".$par." AND childasset=".$chld." relationship=".$k;
					if ($debug == false)
						$db->query($query);
					else
						echo $query;
				}
			}												
		}
		
		$query = "DELETE FROM assetrelationships WHERE parentasset=".$itemid." OR childasset=".$itemid;	
		if ($debug == false)
			$db->query($query);
		else
			echo $query;
			
		$query = "DELETE FROM assetrelclosures WHERE parentasset=".$itemid." OR childasset=".$itemid;
		if ($debug == false)
			$db->query($query);
		else
			echo $query;
			
		$query = "DELETE FROM assetproperties WHERE assetid=".$itemid;
		if ($debug == false)
			$db->query($query);
		else
			echo $query;
			
		$query = "DELETE FROM assetvalues WHERE assetid=".$itemid;					
		if ($debug == false)
			$db->query($query);
		else
			echo $query;
			
		$query = "DELETE FROM datasources WHERE assetid=".$itemid;					
		if ($debug == false)
			$db->query($query);
		else
			echo $query;
			
		$query = "DELETE FROM datalinks WHERE assetid=".$itemid;					
		if ($debug == false)
			$db->query($query);
		else
			echo $query;
			
		$query = "DELETE FROM mediatags WHERE assetid=".$itemid;					
		if ($debug == false)
			$db->query($query);
		else
			echo $query;
			
		$query = "DELETE FROM assets WHERE id=".$itemid;					
		if ($debug == false)
			$db->query($query);
		else
			echo $query;
		
		foreach($toupdate as $tu)
		{
			RebuildObjectCacheByID($tu);
		}
		@unlink($_SERVER['DOCUMENT_ROOT'].'/cache/relationships/'.$id.'.inc');
	}
}

?>