<?php

//Provides an interface to get information about particular relationships.

if (!function_exists("LogEvent"))
{	
	include($installfolder.'/include/logging.php');
}

if (!function_exists("RebuildObjectCacheByID"))
{	
	include($installfolder.'/include/cacheobject.php');
}

class Relationship
{
	public $id;
	public $name;
	public $type;
	public $upname;
	public $downname;
	public $energy;
	public $pass;
	public $options;
	public $colour;
	
	public function LoadFromArray($k,$arr)
	{
		$this->id = $k;
		$this->name = $arr['name'];
		$this->type = $arr['type'];
		$this->upname = $arr['upname'];
		$this->downname = $arr['downname'];
		if (isset($arr['passes']))
			$this->pass = $arr['passes'];
		else
			$this->pass = array();
		$this->energy = false;
		$this->options = array();
		$this->colour = "";
		$this->colour = $arr['colour'];
	}
	
	public function PropertyFlowsThrough($prpid)
	{
		if (in_array($prpid,$this->pass)) return true;
		return false;
	}
	
	public function IsFlowRate()
	{
		if (isset($this->options['flowing'])) return true;
		return false;
	}
	
	public function FormatString($a, $b, $upstream)
	{
		if ($upstream == true)
			return $a.' is '.$this->upname.' '.$b;
		else
			return $a.' '.$this->downname.' '.$b;
	}
}

if (!class_exists('Asset'))
{
	include($installfolder.'/model/asset.php');
}

class Relationships
{
	public static function GetAll()
	{
		return Relationships::GetAllRelationships();
	}

	public static function GetAllRelationships()
	{
		global $sitefolder;
		global $relationships;
		
		if (!isset($relationships))
			require($sitefolder.'/cache/relationships/all.inc');
		
		
		$rels = array();
		foreach($relationships as $k=>$rl)
		{
			$r = new Relationship();
			$r->LoadFromArray($k,$rl);
			$rels[] = $r;
		}
		
		return $rels;
	}

	public static function GetRelationship($id)
	{
		global $sitefolder;
		global $relationships;
		
		if ((!isset($relationships)) || (!isset($relationships[$id])))
		{
			//echo 'Reimporting!';
			require($sitefolder.'/cache/relationships/all.inc');
		}
		
		if (!isset($relationships[$id]))
		{
			//echo 'Relationship '.$id.' Not Available';
			return array();
		}
		$r = new Relationship();
		$r->LoadFromArray($id,$relationships[$id]);
		return $r;
	}
	
	public static function IsAllowed($origin, $destination, $relationship)
	{
		//Step one is to verify that this relationship is ALLOWED.
		
		//echo 'Checking allowance on '.$relationship;
		
		//Firstly, if this relationship is a hierarchy, it may NOT be connected to multiple parents...
		$rel = Relationships::GetRelationship($relationship);
		if (!is_object($rel)) 
		{
			echo 'Couldn\'t Find Relationship '.$relationship;
		}
		if ($rel->type == 'heirachial')
		{
			//Do I have an existing parent on this relationship already?
			$ast = new Asset($destination);
			$rels = $ast->GetRelationships();
			if (isset($rels[$relationship]))
			{
				foreach($rels[$relationship] as $v)
				{
					if ($v['upstream'] == true)
					{
						//echo '[ Barred - This asset is already a '.$relationship.' child of '.$v['target'].' when linking '.$origin.' and '.$destination.' ]';
						return false;
					}
				}
			}
		}
		
		return true;
	}
	
	public static function Add($origin, $destination, $relationship)
	{
		global $db;
		
		if ($origin == 0) return;	
		if ($destination == 0) return;	
		if ($relationship == "") return;	
		
		$bits = explode('-',$relationship);
		
		$newitem = $origin;
		$olditem = $destination;
		
		if (count($bits) > 1)
		{
			if ($bits[1] == 'U')
			{
				$tm = $destination;
				$destination = $origin;
				$origin = $tm;		
			}
		}
						
		$relationship = $bits[0];	
		
		if (!Relationships::IsAllowed($origin, $destination, $relationship))
		{
			return false;
		}
		
		//OK - add basic relationship entry...
		
		LogEvent("asset",$origin,"Link Added To [ass:".$destination."] via [rel:".$relationship."]");
		
		LogEvent("asset",$destination,"Link Added To [ass:".$origin."] via [rel:".$relationship."]");
		
		$debug = false;
		
		$qry = 'INSERT IGNORE INTO assetrelationships (parentasset,childasset,relationship) VALUES ('.$origin.','.$destination.','.$relationship.')';	
		if ($debug == true) 
			echo $qry.'<br/>';
		else
			$db->query($qry);
		
		//Get my effective depth by looking at matching relationships of both parent and child...
		$parentdepth = -9999;
		$childdepth = -9999;
		$newdepth = -9999;
		
		if ($debug == true) echo 'Checking For Existing Depth...<br/>';
		
		$query = $db->query("SELECT depth FROM assetrelclosures WHERE parentasset=".$origin." AND childasset=".$destination." AND relationship=".$relationship);
						
		$existingrecord = false;
		if($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{
			$newdepth = $row['depth'];
			$existingrecord = true;
		}	

		if ($newdepth == -9999)
		{	
			$query = $db->query("SELECT MAX(depth) as mx FROM assetrelclosures WHERE (childasset=".$origin.") AND relationship=".$relationship);
					
			//echo "SELECT MAX(depth) as mx FROM assetrelclosures WHERE (childasset=".$origin.") AND relationship=".$relationship;
					
			if($row = $query->fetch(PDO::FETCH_ASSOC)) 
			{
				$parentdepth = $row['mx'];
				//echo 'New Parent Depth: '.$parentdepth;		
				//if ($parentdepth == "") $parentdepth = -9999;				
			}	
			
			$query = $db->query("SELECT MIN(depth) as mx FROM assetrelclosures WHERE (parentasset=".$destination.") AND relationship=".$relationship);
					
			if($row = $query->fetch(PDO::FETCH_ASSOC)) 
			{
				$childdepth = $row['mx'];
				//if ($childdepth == "") $childdepth = -9999;
			}	
			
			$newdepth = 0;	
		
			if ($childdepth != -9999)
			{
				//This is already attached to other items below, so I'll need to propagate the parent item to all children.
				$newdepth = $childdepth-1;
			}
			
			if ($parentdepth != -9999)
			{
				//My parent is already connected to other things. This doesn't matter so much unless I'm propagating to children, in which case I'll need to proagate these as well...
				$newdepth = $parentdepth+1;
			}
			
			if (($parentdepth != -9999) && ($parentdepth != -9999))
			{
				//Both of the above are true.
				$newdepth = $parentdepth+1;
				$depthnudge = $newdepth - $childdepth;
			}	
			
			if ($debug == true) echo "New Depth: ".$newdepth." (".$parentdepth.", ".$childdepth.")<br/>";
		}	
		else
		{
			if ($debug == true) echo "Existing Closure Record Found - Depth = ".$newdepth.'<br/>';
		}
		
		$downstreamassets = array();
		$downstreamassets[] = $destination;
		
		if ($childdepth != -9999)
		{
			//Get a list of all children of this object...
			if ($debug == true) echo "Trying to load child objects...<br/>";
			if ($debug == true) echo "SELECT childasset FROM assetrelclosures WHERE (parentasset=".$destination.") AND relationship=".$relationship.'<br/>';
			$query = $db->query("SELECT childasset FROM assetrelclosures WHERE (parentasset=".$destination.") AND relationship=".$relationship.' ORDER BY depth');

			$str = "";		
			while($row = $query->fetch(PDO::FETCH_ASSOC)) 
			{
				if ($str != "") $str .= ",";
				$str .= $row['childasset'];
				$downstreamassets[] = $row['childasset'];
				
				if ($debug == true) echo " ".$row['childasset'];
			}	
			if ($debug == true) echo "<br/>";
			
			//Move
			if (isset($depthnudge))
			{						
				if ($depthnudge > 0)
				{
					if ($str != "")
					{
						$qry = "UPDATE assetrelclosures SET depth = depth + ".$depthnudge." WHERE childasset IN (".$str.") AND relationship=".$relationship.' AND depth>'.$newdepth;
						if ($debug == true) 
							echo $qry."<br/>";
						else
							$db->query($qry);
					}
				}			
			}
		}
		
		if ($existingrecord == false)
		{
			$qry = 'INSERT INTO assetrelclosures (parentasset,childasset,relationship,depth) VALUES ('.$origin.','.$destination.','.$relationship.','.($newdepth).')';
			if ($debug == true) 
				echo $qry.'<br/>';
			else
				$db->query($qry);
		}	
		
		//Copy relationship details from the PARENT into the CHILD and all CHILDREN
		foreach($downstreamassets as $dst)
		{
			$qry = "INSERT IGNORE INTO assetrelclosures (parentasset,childasset,relationship,depth) SELECT parentasset,".$dst.",relationship,depth FROM assetrelclosures WHERE childasset=".$origin." AND relationship=".$relationship;
			if ($debug == true) 
				echo $qry."<br/>";
			else
				$db->query($qry);
		}
				
		//$db->query('INSERT INTO assetrelclosures (parentasset,childasset,relationship,depth) VALUES ('.$origin.','.$destination.','.$relationship.','.($maxdepth+1).')');		
		
		//TODO: Urgent - update closures when this item is added as a parent instead of a child.
		
		if(!function_exists("RebuildObjectCacheByID"))
		{
			require($installfolder.'/include/cacheobject.php');
		}
		
		RebuildObjectCacheByID($origin);
		RebuildObjectCacheByID($destination);
		
		return true;
	}

	public static function Remove($asset, $other, $rl)
	{	
		global $db;
			
		if ($asset == 0) return;	
		if ($other == 0) return;	
		if ($rl == "") return;	
			
		$relationship = $rl;
		
		if ($relationship == 0) exit;
		
		//echo 'Beginning Removal of Link Between '.$asset.' and '.$other;
		
		//OK - get the nature of this relationship...
		$othertype = '';
		$query = $db->query("SELECT parentasset,childasset FROM assetrelationships WHERE (( parentasset=".$other." AND childasset=".$asset.") OR ( parentasset=".$asset." AND childasset=".$other.')) AND relationship='.$rl);			
		if($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{
			if ($row['parentasset'] == $other)
				$othertype = 'parent';
			else
				$othertype = 'child';
		}	
		if ($othertype == '') exit;
			
		$debug = false;
		
		$p = $other;
		$c = $asset;
		
		if ($othertype == 'child')
		{
			$p = $asset;
			$c = $other;
		}
		
		//echo 'Asset '.$other.' is a '.$othertype.' of asset '.$asset;
		
		$qry = 'DELETE FROM assetrelationships WHERE relationship='.$relationship.' AND parentasset='.$p.' AND childasset='.$c;	
		if ($debug == true) 
			echo $qry.'<br/>';
		else
			$db->query($qry);
			
		//echo $qry;
			
		$query = $db->query("SELECT depth FROM assetrelclosures WHERE parentasset=".$p." AND childasset=".$c." AND relationship=".$relationship);
					
		if($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{
			$thisdepth = $row['depth'];
		}	
		
		//Get a list of all children of this object...
		if ($debug == true) echo "Trying to load child objects...<br/>";
		
		//echo "SELECT childasset FROM assetrelclosures WHERE parentasset=".$c." AND relationship=".$relationship.' ORDER BY depth'."<br/>";
		
		$str = $c;		
		
		$query = $db->query("SELECT childasset FROM assetrelclosures WHERE parentasset=".$c." AND relationship=".$relationship.' ORDER BY depth');

		$downstreamassets = array();
		while($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{
			if ($str != "") $str .= ",";
			$str .= $row['childasset'];
			$downstreamassets[] = $row['childasset'];
			
			if ($debug == true) echo " ".$row['childasset'];
		}	
		if ($debug == true) echo "<br/>";
		
		//echo count($downstreamassets).' Down Stream Assets Found<br/>';
			
		//Destroy
			
		if ($str != "")
		{
			if (isset($thisdepth))
			{
				$qry = "DELETE FROM assetrelclosures WHERE childasset IN (".$str.") AND relationship=".$relationship." AND depth >= ".$thisdepth;
				if ($debug == true) 
					echo $qry."<br/>";
				else
					$db->query($qry);
			}
		}
		
		$qry = "DELETE FROM assetrelclosures WHERE childasset = ".$c." AND relationship=".$relationship;
		if ($debug == true) 
			echo $qry."<br/>";
		else
			$db->query($qry);
			
		if (!function_exists("RebuildObjectCacheByID"))
		{
			require($_SERVER['DOCUMENT_ROOT'].'/include/cacheobject.php');
		}
		
		RebuildObjectCacheByID($p);
		RebuildObjectCacheByID($c);
		
		if (!function_exists("RebuildObjectClosureTable"))
		{
			require($_SERVER['DOCUMENT_ROOT'].'/include/closuretables.php');
		}
		
		RebuildObjectClosureTable($c,$relationship);
		foreach($downstreamassets as $dsa)
		{
			RebuildObjectClosureTable($dsa,$relationship);
		}
		
		LogEvent("asset",$asset,"Linked Removed From [ass:".$other."] via [rel:".$rl."]");
		
		LogEvent("asset",$other,"Linked Removed From [ass:".$asset."] via [rel:".$rl."]");
	}

	public static function Insert($newid, $parent, $childid, $rel)
	{
		global $db;
		
		$debug = false;
		
		LogEvent("asset",$newid,"Inserted Into [rel:".$rel."] between [ass:".$parent."] and [ass:".$childid."]");
		
		LogEvent("asset",$parent,"[ass:".$newid."] inserted in [rel:".$rel."] before [ass:".$childid."]");
		
		LogEvent("asset",$childid,"[ass:".$newid."] inserted in [rel:".$rel."] after [ass:".$parent."]");
		
		//Step one - grab the EXISTING asset relationships for the item we are replacing...
		$query = $db->query('SELECT * FROM assetrelationships WHERE childasset='.$childid.' AND relationship='.$rel);
		
		//echo 'SELECT * FROM assetrelationships WHERE childasset='.$childid.' AND relationship='.$rel;

		$rec = array();
		while($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{
			$rec[] = $row;
		}	
		
		//Step one-point-five - do the same for the CLOSURE TABLE...
		
		$ctable = array();
		$query = $db->query('SELECT * FROM assetrelclosures WHERE childasset='.$childid.' AND relationship='.$rel);

		while($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{
			$ctable[] = $row;
		}	
		
		//Step two - delete those relationships and replace them with a link to the new item...
		$qry = 'DELETE FROM assetrelationships WHERE childasset='.$childid.' AND relationship='.$rel;
		if ($debug == false)
			$db->query($qry);
		else
			echo $qry;
			
		$qry = 'INSERT INTO assetrelationships (childasset,parentasset,relationship) VALUES ('.$childid.','.$newid.','.$rel.')';
		if ($debug == false)
			$db->query($qry);
		else
			echo $qry;
				
		
		//Step three - re-introduce those relationships into the current item...
		if ($debug == true)
		{
			echo 'Reintroducing Earlier Relationships...<br/>';
		}
		foreach($rec as $r)
		{
			$qry = 'INSERT INTO assetrelationships (childasset,parentasset,relationship) VALUES ('.$newid.','.$r['parentasset'].','.$rel.')';
			if ($debug == false)
				$db->query($qry);
			else
				echo $qry;
		}
		foreach($ctable as $r)
		{
			$qry = 'INSERT INTO assetrelclosures (childasset,parentasset,relationship,depth) VALUES ('.$newid.','.$r['parentasset'].','.$rel.','.$r['depth'].')';
			if ($debug == false)
				$db->query($qry);
			else
				echo $qry;
		}
				
		//Get the depth of this relationship currently...
		$existingdepth = 0;
		$query = $db->query('SELECT depth FROM assetrelclosures WHERE parentasset='.$rec[0]['parentasset'].' AND childasset='.$childid.' AND relationship='.$rel);

		if($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{
			$existingdepth = $row['depth']+1;
		}	
											
		//Get a list of all items under this point...	
		$query = $db->query('SELECT DISTINCT childasset FROM assetrelclosures WHERE relationship='.$rel.' AND depth>='.$existingdepth);

		$children = array();
		while($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{
			$children[] = $row['childasset'];
		}
		
		foreach($children as $child)
		{
			//Push Down
			$qry = 'UPDATE assetrelclosures SET depth = depth+1 WHERE childasset='.$child.' AND depth >= '.$existingdepth.' AND relationship='.$rel;
			if ($debug == false)
				$db->query($qry);
			else
				echo $qry;
			
			//Add New
			$qry = 'INSERT IGNORE INTO assetrelclosures (childasset,parentasset,depth,relationship) VALUES ('.$child.','.$newid.','.$existingdepth.','.$rel.')';
			if ($debug == false)
				$db->query($qry);
			else
				echo $qry;
		}
		
		//Add a closure for the new parent...
		$qry = 'INSERT INTO assetrelclosures (childasset,parentasset,relationship,depth) VALUES ('.$childid.','.$newid.','.$rel.','.($existingdepth+1).')';
		if ($debug == false)
			$db->query($qry);
		else
			echo $qry;
				
		
		if ($debug == true) exit;
								
		require_once($_SERVER['DOCUMENT_ROOT'].'/include/cacheobject.php');

		RebuildObjectCacheByID($childid);
		RebuildObjectCacheByID($newid);
		
		//Final step - rebuild the details of all direct parents, children and the object itself.
		foreach($rec as $r)
		{
			RebuildObjectCacheByID($r['parentasset']);
		}	
	}
	
}

?>