<?php

class Sites
{
	private static $slist = array();

	public static function GetSiteList()
	{
		if (count(self::$slist) != 0) continue;
		
		global $installfolder;
		$sitebase = $installfolder.'/sites';
		foreach (glob($sitebase.'/*',GLOB_ONLYDIR) as $filename) 
		{
			$sitecode = substr($filename,strlen($sitebase)+1);
			
			$a = array();
			
			$i = parse_ini_file($filename.'/config/settings.ini',true);

			if ($sitecode[0] == "_") 
			{
				$a['disabled'] = true;
			}
			$a['name'] = $i['server']['company'];

			//echo 'Saving '.$sitecode;
			$sl[$sitecode] = $a;
		};
		self::$slist = $sl;
	}

	public static function RestoreDB()
	{
		global $db;
		self::GetSiteList();
	}
	
	private static function QueryAcrossAll($qry, $includeme = false)
	{
		global $site;
		global $installfolder;
		global $sitefolder;
		
		$ttl = 0;
		foreach(self::$slist as $code => $info)
		{
			if (($code == $site) && ($includeme == false)) continue;
			
			//Connect To DB...
			include($installfolder.'/sites/'.$code.'/config/dbsettings.php');
			
			$srvr = "localhost";
			if ($dbserver != "") $srvr = $dbserver;
			
			$dbcommandstring = $dbtype.":host=".$srvr.';dbname='.$dbname.";charset=utf8";
			
			if (isset($dboptions))	
				$dbx = new PDO($dbcommandstring,$dbuser,$dbpass,$dboptions);	
			else
				$dbx = new PDO($dbcommandstring,$dbuser,$dbpass);
				
			$query = $dbx->query($qry);
			if ($query === FALSE) 
			{
				$dbx = null;
				continue;
			}
			
			if($row = $query->fetch(PDO::FETCH_ASSOC)) 
			{
				if ($ttl < $row['targ'])
				{
					$ttl = $row['targ'];
				}
			}
			
			$dbx = null;
		}
		include($sitefolder.'/config/dbsettings.php');
		return $ttl;
	}
	
	public static function IsDriverPortUnique($prt)
	{
		if (self::QueryAcrossAll("SELECT COUNT(*) as targ FROM datadriver WHERE port=".$prt) > 0) return false;
		return true;
	}
	
	public static function IsSubPortUnique($prt)
	{
		if (self::QueryAcrossAll("SELECT COUNT(*) as targ FROM profiles WHERE subport=".$prt) > 0) return false;
		return true;
	}
	
	public static function IsReqPortUnique($prt)
	{
		if (self::QueryAcrossAll("SELECT COUNT(*) as targ FROM profiles WHERE subport=".$prt) > 0) return false;
		return true;
	}
	
	public static function GetNextPort()
	{
		$res = self::QueryAcrossAll("SELECT MAX(port) as targ FROM datadriver",true);
		return $res+1;
	}
	
	public static function GetNextSubPort()
	{
		$res = self::QueryAcrossAll("SELECT MAX(subport) as targ FROM profiles",true);
		return $res+1;
	}
	
	public static function GetNextReqPort()
	{
		$res = self::QueryAcrossAll("SELECT MAX(reqport) as targ FROM profiles",true);
		return $res+1;
	}
	
	public static function Report()
	{
		global $db;
		self::GetSiteList();
		
		$clean = true;
		
		$query = $db->query("SELECT * FROM datadriver");
		
		while($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{
			if (!Sites::IsDriverPortUnique($row['port']))
			{
				echo 'Port Conflict Detected With '.$row['name'].' Driver!';
				echo ' Suggest replacing with '.Sites::GetNextPort()."<br/>";
				$clean = false;
			}
		}
		
		$query = $db->query("SELECT * FROM profiles");
		
		while($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{
			if (!Sites::IsSubPortUnique($row['subport']))
			{
				echo 'Subscrption Port Conflict Detected With '.$row['name'].' Profile!';
				echo ' Suggest replacing with '.Sites::GetNextSubPort()."<br/>";
				$clean = false;
			}
			if (!Sites::IsReqPortUnique($row['reqport']))
			{
				echo 'Request Port Conflict Detected With '.$row['name'].' Profile!';
				echo ' Suggest replacing with '.Sites::GetNextReqPort()."<br/>";
				$clean = false;
			}
		}
		
		return $clean;
	}
	
	public static function AutoClean()
	{
		global $db;
		self::GetSiteList();
		
		$query = $db->query("SELECT * FROM datadriver");
		
		$changes = array();
		
		while($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{
			if (!Sites::IsDriverPortUnique($row['port']))
			{
				$changes[] = $row['port'];
				$clean = false;
			}
		}
		if (count($changes) > 0)
		{
			$base = Sites::GetNextPort();
			foreach($changes as $ch)
			{
				$db->query("UPDATE datadriver SET port=".$base." WHERE port=".$ch);
				$db->query("UPDATE datasources SET gatewayport=".$base." WHERE gatewayport=".$ch);
				$base++;
			}
		}
		
		$changesa = array();
		$changesb = array();
		$query = $db->query("SELECT * FROM profiles");
		
		while($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{
			if (!Sites::IsSubPortUnique($row['subport']))
			{
				$changesa[] = $row['subport'];
				$clean = false;
			}
			if (!Sites::IsReqPortUnique($row['reqport']))
			{
				$changesb[] = $row['reqport'];
				$clean = false;
			}
		}
		
		if (count($changesa) > 0)
		{
			$base = Sites::GetNextSubPort();
			foreach($changesa as $ch)
			{
				$db->query("UPDATE profiles SET subport=".$base." WHERE subport=".$ch);
				$base++;
			}
		}
		
		if (count($changesb) > 0)
		{
			$base = Sites::GetNextReqPort();
			foreach($changesa as $ch)
			{
				$db->query("UPDATE profiles SET reqport=".$base." WHERE reqport=".$ch);
				$base++;
			}
		}
	}
}

?>