<?php
/** Reads a spreadsheet file 
* This module reads a CSV file and creates assets, data bindings and relationships based on the content.
**/

	$debugmode = true;

	//Get the file
	if ($_FILES["content"]["error"] > 0) 
	{
		echo "Error: " . $_FILES["content"]["error"] . "<br>";
	}
	else
	{	
		//Sanity-check the file size
		if (($_FILES["content"]["size"] / 1024) > 10240)
		{
			echo 'Error: File To Large';
			unlink($_FILES["content"]["tmp_name"]);
			exit();
		}
		
		//Check for a suitable extension
		$ext = pathinfo ( $_FILES["content"]["name"],PATHINFO_EXTENSION);		
		
		if ((trim(strtolower($ext)) != "txt") && (trim(strtolower($ext)) != "csv"))
		{
			echo 'Error: Incorrect File Extension. Spreadsheets should be uploaded as TXT or CSV files';
			unlink($_FILES["content"]["tmp_name"]);
			exit();
		}
		
		//Copy the file into a local temp folder and remove the original
		$dirname = $sitefolder.'/cache/tmp';
		if (!file_exists($dirname))
		{
			mkdir($dirname);
		}
		
		if (!file_exists($dirname)) mkdir($dirname);
		
		$fname = $dirname. '/ssheet.'.$ext;
		move_uploaded_file($_FILES["content"]["tmp_name"],$fname);
			
		//Begin the import
		echo 'Loading Spreadsheet Contents...<br/><br/>';
		
		$file = fopen($fname,"r");
		$headers = fgetcsv($file);
		
		//Load names of properties and relationships
		$properties = array();
		$query = $db->query('SELECT name,id,typename,options FROM properties ORDER BY name ASC');
	
		while($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{
			$properties[$row['id']]['name'] = $row['name'];
			$properties[$row['id']]['type'] = $row['typename'];
			
			$properties[$row['id']]['defaults'] = array();
			
			$split = explode(':',$row['options']);
			foreach($split as $q)
			{
				$px = explode('=',$q);
				if (count($px) > 1)
				{
					if ($px[0] == 'default')
					{
						$properties[$row['id']]['defaults'] = explode('|',$px[1]);
						//echo 'Found Default for Property '.$row['name'];
						//print_r($properties[$row['id']]['defaults']);
					}
				}
			}
			
			
		}	
		
		$relationships = array();
		$query = $db->query('SELECT name,id FROM relationships ORDER BY name ASC');
	
		while($row = $query->fetch(PDO::FETCH_ASSOC)) 
		{
			$relationships[$row['id']] = $row['name'];
		}	
		
		$namecolumn = -1;
		$prefixcolumn = -1;
		$idcolumn = -1;
		$desccolumn = -1;
		$dscolumn = -1;
		$bindings = array();
		$relbindings = array();
		
		//OK - which headers reflect which properties?
		for($x=0;$x<count($headers);$x++)
		{
			$lc = strtolower($headers[$x]);
			if ($lc == "name") $namecolumn = $x;
			if ($lc == "_prefix") $prefixcolumn = $x;
			if ($lc == "_id") $idcolumn = $x;
			if ($lc == "description") $desccolumn = $x;
			if ($lc == "datasource") $dscolumn = $x;
			foreach($properties as $k => $v)
			{
				if (strtolower($v['name']) == $lc)
				{
					$bnd = array();
					$bnd['propname'] = $v['name'];
					$bnd['propid'] = $k;
					$bnd['default'] = $v['defaults'];
					//echo 'Binding Column '.$x.' To Default...';
					//print_r($bdn['default']);
					
					$bindings[$x] = $bnd;
				}
			}
			foreach($relationships as $k => $v)
			{
				if (strtolower($v) == $lc)
				{
					$bnd = array();
					$bnd['name'] = $v;
					$bnd['id'] = $k;
					$relbindings[$x] = $bnd;
				}
			}
		}
		
		//Was there a 'name' column?
		if ($namecolumn == -1)
		{
			echo 'Your spreadsheet file requires a header row, and one column (ideally the first one) should be called &quot;Name&quot;';
			exit;
		};
		
		$finalbinds = array();
		$assetdetails = array();
		$relations = array();
		
		$indx = 0;
		
		//For each line...
		while(!feof($file))
  		{
			$lines = fgetcsv($file);
			if (trim($lines[$namecolumn]) == "") continue;
			
			if ($prefixcolumn != -1)
			{
				$lines[$namecolumn] = $lines[$prefixcolumn].' '.$lines[$namecolumn];
			}
			echo 'Creating New Asset: ' . $lines[$namecolumn];
			if ($desccolumn != -1) echo ' - '.$lines[$desccolumn];
			echo '<br/>';			
			
			//Establish our new asset
			$as = array();
			$as['name'] = $lines[$namecolumn];
			if ($desccolumn != -1) 
				$as['desc'] = $lines[$desccolumn];
			else
				$as['desc'] = "";
			
			if ($dscolumn != -1)
			{
				//Check for a possible data source...
				if (trim($lines[$dscolumn]) != "")
				{
					echo '   <strong>This is a data source!</strong><br/>';
					$as['ds'] = $lines[$dscolumn];
				}				
			}
			
			//Load property values...
			for($x=0;$x<count($lines);$x++)
			{
				if (trim($lines[$x]) == "") continue;
				
				if (isset($relbindings[$x]))
				{					
					echo '  Related via '.$relbindings[$x]['name'].' to '.$lines[$x].'<br/>';
					if (!isset($relations[$indx])) $relations[$indx] = array();
					$lnk = array();
					$lnk['target'] = $lines[$x];
					$lnk['relationship'] = $relbindings[$x]['id'];
					$relations[$indx][] = $lnk;
					continue;
				}
				
				if (isset($bindings[$x]))
				{
					//Copy this value 
					echo '  Property '.$bindings[$x]['propname'].': '.$lines[$x].'<br/>';										
					$parts = explode('|',$lines[$x]);
					
					$typename = $properties[$bindings[$x]['propid']]['type'];								
					$node = "";
					$fn = $_SERVER['DOCUMENT_ROOT'].'/modules/inputs/'.strtolower($typename).'.php';
					if (file_exists($fn))
					{
						include($fn);				
					}
					
					$vx=0;
						
					for($vx=count($parts);$vx<count($inputs);$vx++)
					{
						if (isset($bindings[$x]['default'][$vx]))
						{
							
							$parts[$vx] = $bindings[$x]['default'][$vx];
						}
						else
						{
							if (isset($inputs[$vx]['default']))
							{
								
								$parts[$vx] = $inputs[$vx]['default'];
							}
							else
							{
								$parts[$vx] = "";
							}
						}
					}
								
					for($q=0;$q<count($parts);$q++)
					{
						//Check for data links...
						if ($parts[$q] == "") continue;
						if ($parts[$q][0] == '>')
						{
							$bits = explode('>',$parts[$q]);
							if (count($bits) >= 2) 
							{
								//OK - before we continue find the name of this node (via the input file)	
								$node = $inputs[$q]['name'];									
								
								if ($debugmode == true)
								{
									echo '&nbsp;&nbsp;&nbsp;Asset Type: '.$typename.'<br/>';
								
									echo '&nbsp;&nbsp;&nbsp;Binding To '.$bits[1].' ( '.$node.' ) - ';
									echo ' to address '.$bits[2];
									if (isset($bits[3])) echo ', Transform '.$bits[3];
									echo '<br/>';
								}
								
								$b = array();
								$b['source'] = $bits[1];
								$b['node'] = $node;
								$b['address'] = $bits[2];
								if (count($bits) >= 4) 
									$b['transform'] = $bits[3];
								else
									$b['transform'] = "";
									
								$b['asset'] = $indx;
								$b['property'] = $bindings[$x]['propid'];
								$finalbinds[] = $b;
								
								$parts[$q] = "";
							}
						}
						
						//Check for exotic data types...
						if (isset($inputs[$q]['type']))
						{
							$type = $inputs[$q]['type'];
							
							if ($type == 'relno')
							{
								$found = false;
								$cmp = strtolower($parts[$q]);
								foreach($relationships as $id => $rl)
								{
									if (strtolower($rl) == $cmp)
									{
										$parts[$q] = $id;
										$found = true;
										break;
									}
								}
								
								if ($found == false)
								{
									echo 'Unable to locate a matching relationship!<br/>';
									$parts[$q] = "";
								}
							}
						}
						
					}
					$vlu = implode('|',$parts);
					$as[$bindings[$x]['propid']] = $vlu;
				}
			}
			$assetdetails[$indx] = $as;
			$indx++;
		}
		fclose($file);
		
		$debug = true;
		
		//print_r($_REQUEST);
		
		//For each discovered asset...
		foreach($assetdetails as $ky => $ass)
		{
			//Create the asset itself...
			$qry = "INSERT INTO assets (name,descript) VALUES (".$db->quote($ass['name']).",".$db->quote($ass['desc']).")";
			
			if ($debug == false)
				$db->query($qry);
			else
				echo $qry.'<br/>';
											
			$id = $db->lastInsertId();
				
			$assetdetails[$ky]['id'] = $id;										
				
			//Create property records				
			foreach($ass as $k => $v)
			{
				if ($k == "name") continue;
				if ($k == "desc") continue;
				if ($k == "ds")
				{
					//This is a data source! Create it here.
					$bits = explode('|',$v);
					$qry = "INSERT INTO datasources (assetid, driver, address) VALUES (".$id.",".$db->quote($bits[0]).",".$db->quote($bits[1]).")";
			
					if ($debug == false)
						$db->query($qry);
					else
						echo $qry.'<br/>';
				}
				
				/*$qry = "INSERT INTO assetproperties (assetid, propertyid) VALUES (".$id.",".$k.")";
			
				if ($debug == false)
					$db->query($qry);
				else
					echo $qry.'<br/>';*/
					
				//echo 'Index '.$k.' has value '.$v;
					
				if (($v == "") || ($v[0] != ">"))
				{
					$qry = "INSERT INTO assetvalues (assetid, propertyid, value) VALUES (".$id.",".$k.",".$db->quote($v).")";
				
					if ($debug == false)
						$db->query($qry);
					else
						echo $qry.'<br/>';
				}								
			}						
		}
		
		//Create all data links
		foreach($finalbinds as $fb)
		{
			$src = $fb['source'];
			unset($sourceid);
			
			//Does this exist in the imported data?
			foreach($assetdetails as $ad)
			{
				if ($ad['name'] == $src)
				{
					$sourceid = $ad['id'];
				}
			}
			
			if (!isset($sourceid))
			{
				//No source found - check database
				$query = $db->query('SELECT name,id FROM assets WHERE name='.$db->quote($src));
	
				if($row = $query->fetch(PDO::FETCH_ASSOC))
				{
					$sourceid = $row['id'];
				}	
			}
			
			if (isset($sourceid))
			{
				$qry = "INSERT INTO datalinks (sourceid,assetid, propertyid, node, address, transform) VALUES (".$sourceid.",".$assetdetails[$fb['asset']]['id'].",".$fb['property'].",".$db->quote($fb['node']).",".$db->quote($fb['address']).",".$db->quote($fb['transform']).")";
			
				if ($debug == false)
					$db->query($qry);
				else
					echo $qry.'<br/>';
			}
		}
		
		//Create all relationships...
		$toprocess = array();
		
		foreach($relations as $k => $v)
		{
			foreach($v as $itm)
			{
				echo $assetdetails[$k]['name'].' is a '.$itm['relationship'].' child of '.$itm['target'].'<br/>';
				
				if ($itm['relationship'] == $_REQUEST['parentrelationship'])
				{
					$assetdetails[$k]['hasbase'] = true;
				}
				//$relations[$k]['queue'] = 0;

				//Check to see if this links to an asset that doesn't exist yet...				
				$found = false;
				foreach($assetdetails as $idx => $ad)
				{
					if ($ad['name'] == $itm['target'])
					{
						$found = true;
						$toprocess[$k]['internal'] = $idx;
						$toprocess[$k]['queue'] = 1;
						break;
					}
				}				
				
				if ($found == false)
				{
					//OK - find the real asset...					
					$query = $db->query('SELECT id FROM assets WHERE name='.$db->quote($itm['target']));
				
					if($row = $query->fetch(PDO::FETCH_ASSOC)) 
					{
						//Bingo - this is attached to an existing database asset! Link it, and use it as the root of the next phase...
						//$relations[$k]['queue'] = 1;
						$_REQUEST['newreltarget'] = $assetdetails[$k]['id'];
						$_REQUEST['origin'] = $row['id'];
						$_REQUEST['relationship'] = $itm['relationship']."-D";									
						if ($debug == false) 
							require($installfolder.'/assets/link.php');
						else
							echo 'Linking '.$_REQUEST['origin'].' and '.$_REQUEST['newreltarget'].'<br/>';
					}	
					else
						echo 'Invalid Relationship In '.$assetdetails[$k]['name'].' - Can Not Find An Asset Named '.$itm['target'].'<br/>';
				}							
			}
		}
				
		//Try to get a rough sequence to avoid excessive database updates
		$depth = 0;
		$changes = true;
		while($changes == true)
		{
			$changes = false;
			$thispass = array();
			
			foreach($toprocess as $k => $tp)
			{
				if (!isset($toprocess[$tp['internal']])) continue;
				if ($tp['queue'] <= $toprocess[$tp['internal']]['queue'])
				{
					$tp['queue'] = $toprocess[$tp['internal']]['queue'] + 1;
					$changes = true;
				}
			}
			
			$depth++;
			if ($depth > 10)
			{
				$err = true;
				break;
			}
		}
		
		if (isset($err))
		{
			echo 'Possible Circular Reference';
		}
		
		//Save new relationships
		foreach($toprocess as $tp)
		{
			foreach($relations[$tp['internal']] as $itm)
			{
				$_REQUEST['newreltarget'] = $assetdetails[$k]['id'];
				$_REQUEST['origin'] = $assetdetails[$tp['internal']]['id'];
				$_REQUEST['relationship'] = $itm['relationship']."-D";									
				if ($debug == false) 
					require($installfolder.'/assets/link.php');
				else
					echo 'Linking '.$_REQUEST['origin'].' and '.$_REQUEST['newreltarget'].'<br/>';
			}
		}
		
		foreach($assetdetails as $ad)
		{
			if (!isset($ad['hasbase']))
			{
				$_REQUEST['newreltarget'] = $ad['id'];
				$_REQUEST['origin'] = $_REQUEST['parentbase'];
				$_REQUEST['relationship'] = $_REQUEST['parentrelationship']."-D";									
				if ($debug == false) require($_SERVER['DOCUMENT_ROOT'].'/assets/link.php');
			}
		}
		
		echo 'Import Complete!';
	}
?> 