<?php

include($installfolder.'/include/simpletest/autorun.php');
include($installfolder.'/model/asset.php');
require_once($installfolder.'/include/creator.php');
require_once($installfolder.'/include/deletor.php');

function CheckAssetExists($name)
{
	global $db;
	
	$query = $db->query("SELECT id FROM assets WHERE name='".$name."'");

	$id = -1;
	$already = false;
	while($row = $query->fetch(PDO::FETCH_ASSOC)) 
	{
		if ($already == false)
		{
			$already = true;
			$id = $row['id'];
		}
		else
		{
			echo 'Excessive Results!';
			$id = -1;
			return false;
		}
	}	
	
	return $id;
}

function CheckDirectRelationship($assetid, $toid)
{
	return true;
}

function CheckClosureTable($assetid, $list)
{
	return true;
}

class TestAssetCreationFunctions extends UnitTestCase
{	
	function testLoopPrevention() {    
	   
		global $db; 
		require_once($_SERVER['DOCUMENT_ROOT'].'/include/creator.php');
		require_once($_SERVER['DOCUMENT_ROOT'].'/model/relationship.php');
		
		echo 'Relationships - Single-Point Hierarchy Test<br/>';
        
		$upper = CheckAssetExists('First Asset');
		$this->assertNotEqual($upper,-1);
		
		$lower = CheckAssetExists('Third Asset');
        $this->assertNotEqual($lower,-1);	
		
		//Try to establish a relationship between the THIRD and FIRST assets (cause a circular reference)
		$res = Relationships::Add($upper,$lower,2);
		$this->assertEqual($res,false);
	}
	
	//This removes an asset from the TAIL of a relationship sequence...
	function testSimpleRelRemoval() {    
	   
		global $db; 
		//require_once($_SERVER['DOCUMENT_ROOT'].'/include/creator.php');
		require_once($_SERVER['DOCUMENT_ROOT'].'/model/relationship.php');
		require_once($_SERVER['DOCUMENT_ROOT'].'/model/asset.php');
		
		echo 'Relationships - End-Of-Sequence Removal<br/>';
        
		$upper = CheckAssetExists('Third Asset');
		$this->assertNotEqual($upper,-1);
		
		$lower = CheckAssetExists('Fourth Asset');
        $this->assertNotEqual($lower,-1);	
		
		//Kill off relationship between Third and Fourth asset...
		
		Relationships::Remove($upper,$lower,2);
		
		$ass = new Asset($lower);
		$rel = $ass->GetRelationships();
		$this->assertEqual(count($rel),0);
		
		$cnt = -1;
		$query = $db->query("SELECT COUNT(*) as cnt FROM assetrelclosures WHERE childasset=".$lower." AND relationship=2");
		if ($query !== FALSE)
		{
			if($row = $query->fetch(PDO::FETCH_ASSOC)) 
			{
				$cnt = $row['cnt'];
			}
		}
		
		$this->assertEqual($cnt,0);
	}
	
	function testReaddRemoved() {    
	   
		global $db; 
		//require_once($_SERVER['DOCUMENT_ROOT'].'/include/creator.php');
		require_once($_SERVER['DOCUMENT_ROOT'].'/model/relationship.php');
		require_once($_SERVER['DOCUMENT_ROOT'].'/model/asset.php');
        
		echo 'Relationships - End-Of-Sequence Readdition<br/>';
		
		$upper = CheckAssetExists('Third Asset');
		$this->assertNotEqual($upper,-1);
		
		$lower = CheckAssetExists('Fourth Asset');
        $this->assertNotEqual($lower,-1);	
		
		//Re-Add the relationship between the two points
		$res = Relationships::Add($upper,$lower,2);
		
		$ass = new Asset($lower);
		$rel = $ass->GetRelationships();
		
		$this->assertEqual($res,true);
		$this->assertEqual(count($rel),1);
		
		$cnt = -1;
		$query = $db->query("SELECT COUNT(*) as cnt FROM assetrelclosures WHERE childasset=".$lower." AND relationship=2");
		if ($query !== FALSE)
		{
			if($row = $query->fetch(PDO::FETCH_ASSOC)) 
			{
				$cnt = $row['cnt'];
			}
		}
		
		$this->assertEqual($cnt,5);
	}
	
	function testMiddleRemoval() {    
	   
		global $db; 
		//require_once($_SERVER['DOCUMENT_ROOT'].'/include/creator.php');
		require_once($_SERVER['DOCUMENT_ROOT'].'/model/relationship.php');
		require_once($_SERVER['DOCUMENT_ROOT'].'/model/asset.php');
		
		echo 'Relationships - Mid-Sequence Removal<br/>';
        
		$upper = CheckAssetExists('First Asset');
		$this->assertNotEqual($upper,-1);
		
		$lower = CheckAssetExists('Second Asset');
        $this->assertNotEqual($lower,-1);	
		
		//Remove the relationship between the two points
		Relationships::Remove($upper,$lower,2);
		
		$ass = new Asset($lower);
		$rel = $ass->GetRelationships();
		
		//$this->assertEqual($res,true);
		$this->assertEqual(count($rel),1);
		
		$cnt = -1;
		$query = $db->query("SELECT COUNT(*) as cnt FROM assetrelclosures WHERE childasset=".$lower." AND relationship=2");
		if ($query !== FALSE)
		{
			if($row = $query->fetch(PDO::FETCH_ASSOC)) 
			{
				$cnt = $row['cnt'];
			}
		}
		
		$this->assertEqual(0+$cnt,0);
		
		$cnt = -1;
		$query = $db->query("SELECT COUNT(*) as cnt FROM assetrelclosures WHERE childasset=".$upper." AND relationship=2");
		if ($query !== FALSE)
		{
			if($row = $query->fetch(PDO::FETCH_ASSOC)) 
			{
				$cnt = $row['cnt'];
			}
		}
		
		$this->assertEqual(0+$cnt,2);
	}
	
	function testMiddleReinsert() {    
	   
		global $db; 
		//require_once($_SERVER['DOCUMENT_ROOT'].'/include/creator.php');
		require_once($_SERVER['DOCUMENT_ROOT'].'/model/relationship.php');
		require_once($_SERVER['DOCUMENT_ROOT'].'/model/asset.php');
		
		echo 'Relationships - Mid-Sequence Readdition<br/>';
        
		$upper = CheckAssetExists('First Asset');
		$this->assertNotEqual($upper,-1);
		
		$lower = CheckAssetExists('Second Asset');
        $this->assertNotEqual($lower,-1);	
		
		//Remove the relationship between the two points
		$res = Relationships::Add($upper,$lower,2);
		
		$ass = new Asset($lower);
		$rel = $ass->GetRelationships();
		
		$this->assertEqual($res,true);
		$this->assertEqual(count($rel),1);
		
		$cnt = -1;
		$query = $db->query("SELECT COUNT(*) as cnt FROM assetrelclosures WHERE childasset=".$lower." AND relationship=2");
		if ($query !== FALSE)
		{
			if($row = $query->fetch(PDO::FETCH_ASSOC)) 
			{
				$cnt = $row['cnt'];
			}
		}
		
		$this->assertEqual(0+$cnt,3);
		
		$cnt = -1;
		$query = $db->query("SELECT COUNT(*) as cnt FROM assetrelclosures WHERE childasset=".$upper." AND relationship=2");
		if ($query !== FALSE)
		{
			if($row = $query->fetch(PDO::FETCH_ASSOC)) 
			{
				$cnt = $row['cnt'];
			}
		}
		
		$this->assertEqual(0+$cnt,2);
	}
	
	function testTagAttach() {    
	   
		
	}
}

?>